"""
Python setup file for the Clothoids C++ library. It uses the SWIG interface to
generate the Python bindings for the C++ library.

Authors:
 - Sebastiano Taddei
 - Gabriele Masina
"""

import os
from setuptools import setup, Extension

dirname = os.path.dirname(os.path.abspath(__file__))

# Define the include directories
include_dirs = [
    dirname,
    os.path.join(dirname, "lib", "include"),
    os.path.join(dirname, "lib3rd", "include"),
]

# Define the library directories
library_dirs = [
    os.path.join(dirname, "lib", "lib"),
    os.path.join(dirname, "lib3rd", "lib"),
]

# Define the libraries
library_names = []
prefix = "lib"
for library_dir in library_dirs:
    # Get the filenames in the library directory
    filenames = os.listdir(library_dir)

    # For each filename, remove the prefix and suffix
    for filename in filenames:
        # Remove the file extension
        root = os.path.splitext(filename)[0]

        if root.startswith(prefix):
            root = root[len(prefix) :]

        library_names.append(root)

clothoids_module = Extension(
    "_Clothoids",
    sources=[os.path.join("src_py", "Clothoids.i")],
    include_dirs=include_dirs,
    library_dirs=library_dirs,
    libraries=library_names,
    swig_opts=["-c++"],
    extra_compile_args=["-std=c++11"],
)

setup(
    ext_modules=[clothoids_module],
    py_modules=["Clothoids"],
    package_dir={"": "src_py"},
)



import Clothoids
import numpy as np
import matplotlib.pyplot as plt

# ------------------------------------------------
# Example: create and evaluate a clothoid curve
# ------------------------------------------------

curve = Clothoids.ClothoidCurve("curve")
x_ini     = 0.0  # Initial x-coordinate (m)
y_ini     = 0.0  # Initial y-coordinate (m)
theta_ini = 0.0  # Initial absolute orientation (rad)
x_fin     = 4.0  # Final x-coordinate (m)
y_fin     = 4.0  # Final y-coordinate (m)
theta_fin = 0.0  # Final absolute orientation (rad)
curve.build_G1(x_ini, y_ini, theta_ini, x_fin, y_fin, theta_fin)

s_values = np.arange(0, curve.length(), 0.01, dtype=np.float64)
points = np.zeros((s_values.size, 2))
curvatures = np.zeros(s_values.size, dtype=np.float64)

for i in range(s_values.size):
    points[i, :] = curve.eval(s_values[i])
    curvatures[i] = curve.kappa(s_values[i])

# plot the x,y points of the clothoid curve
plt.figure(figsize=(8, 6))
plt.title("Clothoid Curve Evaluation")
plt.plot(points[:, 0], points[:, 1])
plt.xlabel("x (m)")
plt.ylabel("y (m)")
plt.grid()
plt.axis('equal')
plt.show()

# plot the curvature values
plt.figure(figsize=(8, 6))
plt.title("Clothoid Curvature Evaluation")
plt.plot(s_values, curvatures, 'bo', markersize=2)
plt.xlabel("s (m)")
plt.ylabel("Curvature (1/m)")
plt.grid()
plt.show()

# ------------------------------------------------
# Example: create and evaluate a clothoid list
# ------------------------------------------------

curve_list = Clothoids.ClothoidList("curve_list")
interp_point_x = np.array([0.0, 1.0, 1.0, 0.0], dtype=np.float64)
interp_point_y = np.array([0.0, 0.0, 1.0, 1.0], dtype=np.float64)
curve_list.build_G1(interp_point_x, interp_point_y)

s_values = np.arange(0, curve_list.length(), 0.01, dtype=np.float64)
points = np.zeros((s_values.size, 2))

for i in range(s_values.size):
    points[i, :] = curve_list.eval(s_values[i])

# plot the x,y points of the clothoid list
plt.figure(figsize=(8, 6))
plt.title("Clothoid List Evaluation")
plt.xlabel("x (m)")
plt.ylabel("y (m)")
plt.grid()
plt.axis('equal')
plt.plot(points[:, 0], points[:, 1])
plt.show()
